import * as line from "@line/bot-sdk";
import dayjs from "dayjs";
import DBTools from "./DBTools";
import { NumberEx } from "./Engine/Number/NumberEx";

/**
 * Badminton
 */
export default class Badminton {

	//#region Custom

	public static async Run(msg: string[]): Promise<line.Message | line.Message[]> {
		let message: line.Message | line.Message[] = null;
		const instruction: string = msg[1];
		switch (instruction) {
			case "查詢參加人員": {
				const text: string = await this.searchPlay(+msg[2]);
				message = {
					type: "text",
					text: text
				};
				break;
			}

			case "查詢對戰人員": {
				message = await this.searchMatch(+msg[2]);
				break;
			}

			// case "查詢戰績": {
			// 	const queryresp: any = await this.searchPlay(value);
			// 	message.text = "Success";
			// 	break;
			// }

			case "輸入名單": {
				message = await this.inputList(msg);
				break;
			}

			default:
				break;
		}
		return message;
	}

	private static async searchPlay(date: number): Promise<string> {
		let resp: string = "";
		const query: string = String.Format("SELECT * FROM `badminton` WHERE `time` = '{0}' LIMIT 1;", date);
		const queryresp: any[] = await DBTools.Query(query);
		if (queryresp.length > 0) {
			resp = queryresp[0].personnel;
		} else {
			resp = "這天沒有對戰資料";
		}
		return resp;
	}

	private static async searchMatch(date: number): Promise<line.Message | line.Message[]> {
		let resp: line.Message | line.Message[] = {
			type: "text",
			text: "這天沒有對戰資料"
		};
		const query: string = String.Format("SELECT * FROM `badminton` WHERE `time` = '{0}' LIMIT 1;", date);
		const queryresp: any[] = await DBTools.Query(query);
		if (queryresp.length > 0) {
			const matchData_Str: string = queryresp[0].battlecombination;
			if (matchData_Str) {
				const matchData: any = JSON.parse(matchData_Str);
				resp = this.getMatch_LineFlex(date.toString(), matchData);
			}
		}
		return resp;
	}

	/**
	 * 輸入名單
	 * @param msg
	 */
	private static async inputList(msg: any[]): Promise<line.Message | line.Message[]> {
		const date: number = +msg[2];
		let personnelDatas: any[] = [];
		for (let i: number = 3; i < msg.length; i++) {
			const personnelData: any[] = JSON.parse(msg[i]);
			personnelDatas.push(personnelData);
		}
		const personnel_str: string = JSON.stringify(personnelDatas);
		const query: string = String.Format("INSERT INTO `badminton` (time,personnel) VALUES ({0},'{1}') ON DUPLICATE KEY UPDATE personnel='{2}';"
			, date, personnel_str, personnel_str);
		await DBTools.Query(query);
		return await this.listMatch(date, personnelDatas, personnel_str);
	}

	private static async listMatch(date: number, personnelDatas: any[], personnel_str: string): Promise<line.Message | line.Message[]> {
		let man: string[] = [];
		let woman: string[] = [];
		for (let i: number = 0; i < personnelDatas.length; i++) {
			const personnelData: any[] = personnelDatas[i];
			if (personnelData[0]) {
				man.push(personnelData[1]);
			} else {
				woman.push(personnelData[1]);
			}
		}
		// 打亂
		man.sort(function (): number {
			return (0.5 - Math.random());
		});
		woman.sort(function (): number {
			return (0.5 - Math.random());
		});
		let diffCount: number = Math.abs(NumberEx.minus(man.length, woman.length));
		if (man.length !== woman.length && diffCount >= 2) {
			const needDiffCount: number = NumberEx.divide(diffCount, 2);
			if (man.length > woman.length) {
				for (let i: number = 0; i < needDiffCount; i++) {
					woman.push(man.shift());
				}
			} else {
				for (let i: number = 0; i < needDiffCount; i++) {
					man.push(woman.shift());
				}
			}
		}
		const totalRound: number = 3;
		const totalCount: number = man.length > woman.length ? man.length : woman.length;
		let lastmatch: Object = {};
		let check: string[] = [];
		let m_temp: number[] = [];
		let w_temp: number[] = [];
		for (let i: number = 0; i < totalCount; i++) {
			m_temp.push(i);
			w_temp.push(i);
		}
		for (let i: number = 0; i < totalRound; i++) {
			const [match_num, check_temp] = this.getMatch(totalCount, check, m_temp, w_temp);
			let match: string[][] = [];
			check = check_temp;
			for (let j: number = 0; j < match_num.length; j++) {
				const [m, w] = match_num[j];
				const man_str: string = man[m] ?? "那個";
				const woman_str: string = woman[w] ?? "那個";
				match.push([man_str, woman_str]);
			}
			lastmatch[i] = match;
		}
		// console.log(`manCount: ${man.length}, womanCount: ${woman.length}`);
		const result: string = JSON.stringify(lastmatch);
		const query: string = String.Format("INSERT INTO `badminton` (time,personnel,battlecombination) VALUES ({0},'{1}','{3}') ON DUPLICATE KEY UPDATE battlecombination='{2}';"
			, date, personnel_str, result, result);
		// console.log(`query: ${query}`);
		await DBTools.Query(query);
		let message: line.Message | line.Message[] = this.getMatch_LineFlex(date.toString(), lastmatch);
		return message;
	}

	/**
	 * 取得名單
	 * @param totalCount
	 * @param check
	 * @param m_temp
	 * @param w_temp
	 * @param repeatCount
	 */
	private static getMatch(totalCount: number, check: string[], m_temp: number[], w_temp: number[]): [number[][], string[]] {
		const repeatMaxCount: number = 10;
		let check_temp: string[] = [].concat(check);
		const match: number[][] = [];
		let m: number[] = [].concat(m_temp);
		let w: number[] = [].concat(w_temp);
		// 打亂
		m.sort(function (): number {
			return (0.5 - Math.random());
		});
		w.sort(function (): number {
			return (0.5 - Math.random());
		});
		for (let j: number = 0; j < totalCount; j++) {
			for (let k: number = 0; k < repeatMaxCount; k++) {
				const check_str: string = `m${m[0]}w${w[0]}`;
				if (check.includes(check_str) && j !== totalCount - 1) {
					w.sort(function (): number {
						return (0.5 - Math.random());
					});
					continue;
				}
				check_temp.push(check_str);
				match.push([m.shift(), w.shift()]);
				break;
			}
		}
		return [match, check_temp];
	}

	/**
	 * 取得名單Line模板
	 * @param date
	 * @param matchs
	 */
	private static getMatch_LineFlex(date: string, matchs: Object): line.Message | line.Message[] {
		const contents: any[] = [];
		const dateTime_Str: string = `${date.substring(0, 4)}-${date.substring(4, 6)}-${date.substring(6, 8)}`;
		let dateTime: number = new Date(dateTime_Str).getTime();
		let date_Str: string = dayjs(Math.floor(dateTime)).format("YYYY-MM-DD");
		for (let i: number = 0, matchKey: string[] = Object.keys(matchs); i < matchKey.length; i++) {
			const key: string = matchKey[i];
			const match: string[] = matchs[key];
			let boxs: any[] = [];
			const bubble: any = {
				"type": "bubble",
				"size": "micro",
				"body": {
					"type": "box",
					"layout": "vertical"
				},
				"styles": {
					"footer": {
						"separator": true
					}
				}
			};

			const bubble_contents: Object = [
				{
					"type": "text",
					"text": "勝皇羽球團",
					"weight": "bold",
					"color": "#1DB446",
					"size": "sm"
				},
				{
					"type": "text",
					"text": date_Str,
					"weight": "bold",
					"size": "xl",
					"margin": "sm"
				},
				{
					"type": "text",
					"text": `第${i + 1}輪`,
					"weight": "bold",
					"size": "xxl",
					"margin": "sm"
				},
				{
					"type": "separator",
					"margin": "md"
				},
				{
					"type": "box",
					"layout": "horizontal",
					"contents": [
						{
							"type": "text",
							"text": "一號隊友",
							"size": "sm",
							"flex": 0
						},
						{
							"type": "text",
							"text": "二號隊友",
							"size": "sm",
							"align": "center"
						}
					],
					"margin": "md"
				},
				{
					"type": "separator",
					"margin": "sm"
				},
				{
					"type": "box",
					"layout": "vertical",
					"contents": boxs,
					"spacing": "sm",
					"margin": "md"
				}];
			for (let j: number = 0; j < match.length; j++) {
				const box: any = {
					"type": "box",
					"layout": "horizontal",
					"contents": [
						{
							"type": "text",
							"text": match[j][0],
							"size": "sm",
							"flex": 0
						},
						{
							"type": "text",
							"text": match[j][1],
							"size": "sm",
							"align": "center"
						}
					]
				};
				boxs.push(box);
			}
			bubble.body["contents"] = bubble_contents;
			contents.push(bubble);
		}
		const messages: line.Message[] = [
			{
				"type": "flex",
				"altText": "對戰組合名單",
				"contents":
				{
					"type": "carousel",
					"contents": contents
				}
			}
		];
		return messages;
	}

	//#endregion
}
