import * as line from "@line/bot-sdk";
import Badminton from "./Badminton";
import OpenAI from "./OpenAI";

/**
 * Message
 */
export default class MessageClass {

	//#region private

	private bot: line.Client = null;

	//#endregion

	//#region Lifecycle

	constructor(bot: line.Client) {
		this.bot = bot;
	}

	//#endregion

	//#region Custom

	public async Message(event: any): Promise<void> {
		switch (event.message.type) {
			case "text": {
				return await this.Text(event);
			}

			case "sticker": {
				return await this.Sticker(event);
			}

			default:
				break;
		}
	}

	public async Text(event: any): Promise<void> {
		switch (event.source.type) {
			case "user": {
				return await this.User(event);
			}

			case "group":
			default:
				break;
		}
	}

	public async Sticker(event: any): Promise<any> {
		switch (event.source.type) {
			case "user": {
				let replyMsg: string = `https://liff.line.me/1657715144-m4W6lyjL?type=sticker&stk=noanim&sid=${event.message.stickerId}&pkg=${event.message.packageId}`;
				const resp: line.Message = {
					type: "text",
					text: replyMsg
				};
				return this.bot.replyMessage(event.replyToken, resp);
			}

			case "group":
			default: {
				return Promise.resolve(null);
			}
		}
	}

	public async User(event: any): Promise<any> {
		let userId: string = event.source.userId;
		let replyMsg: string = event.message.text;
		// let displayName = "";
		// let profile = await this.bot.getProfile(userId);
		// if (profile) {
		// 	displayName = profile.displayName;
		// }
		// 豬喵 特別功能
		if ([process.env.toJianMiau, process.env.toZhuHan].includes(userId)) {
			/** 訊息 */
			let msg: string[] = event.message.text.split(" ");

			if (userId === process.env.toJianMiau) {
				// 建喵 指令功能
				/** 指令 */
				let instruction: string = msg[0];
				switch (instruction) {
					case "羽球": {
						const messages: line.Message | line.Message[] = await Badminton.Run(msg);
						if (messages) {
							return this.bot.replyMessage(event.replyToken, messages);
						}
					}

					default: {
						const response: string = await OpenAI.RunOpenAI(replyMsg);
						replyMsg = response;
						break;
					}
				}
			} else {
				// 豬涵 指令功能
				const response: string = await OpenAI.RunOpenAI(replyMsg);
				replyMsg = response;
			}
		}

		return this.bot.replyMessage(event.replyToken, {
			type: "text",
			text: replyMsg
		});
		// return this.bot.replyMessage(event.replyToken, {
		// 	type: "text",
		// 	text: event.message.text
		// });
	}

	public async Group(event: any): Promise<any> {
		let groupId: string = event.source.groupId;
		// 羽球 特別功能
		if ([process.env.toBadminton].includes(groupId)) {
			/** 訊息 */
			let msg: string[] = event.message.text.split(" ");
			/** 指令 */
			let instruction: string = msg[0];
			switch (instruction) {
				case "羽球": {
					const messages: line.Message | line.Message[] = await Badminton.Run(msg);
					if (messages) {
						return this.bot.replyMessage(event.replyToken, messages);
					}
				}

				default: {
					break;
				}
			}
		}
	}

	// Group(event) {
	// 	switch (event.source.groupId) {
	// 		case process.env.toYoutube: {
	// 			let messagereplace = event.message.text;
	// 			messagereplace.replace("【IFTTT】 \n", "");
	// 			let replyMsg = messagereplace;
	// 			let res_toUniversity = this.bot.pushMessage(process.env.toUniversity, replyMsg);
	// 			let res_toApex = this.bot.pushMessage(process.env.toApex, replyMsg);
	// 			break;
	// 		}

	// 		default:
	// 			break;
	// 	}
	// }

	//#endregion
}
